//
//  NXDataWrapper.h
//  replicant-objc
//
//  Created by Maksim Tyrtyshny on 7/23/12.
//  Copyright (c) 2012 Nullsoft, Inc. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "NXFileStatWrapper.h"

typedef struct nx_data_struct_t *nx_data_fwd_t;

@interface NXDataWrapper : NSObject
{
@private
	nx_data_fwd_t nxData;
}

+ (id)data;

+ (id)dataWithError:(NSError **)error;

+ (id)dataWithBytes:(const void *)bytes  
			 length:(size_t)length
			  error:(NSError **)error;

+ (id)dataWithSize:(size_t)size  
			 error:(NSError **)error;


+ (id)dataWithNSData:(NSData *)nsData
			mimeType:(NSString *)mimeType
			   error:(NSError **)error;


+ (id)dataWithContentsOfURL:(NSURL *)aURL  
					  error:(NSError **)error;

+ (id)dataWithNXData:(nx_data_fwd_t)nxData  
			   error:(NSError **)error;

+ (id)dataWithNXDataWrapper:(NXDataWrapper *)nxDataWrapper  
					  error:(NSError **)error;

- (id)init;
- (id)initWithError:(NSError **)error;

- (id)initWithBytes:(const void *)bytes  
			 length:(size_t)length
			  error:(NSError **)error;

- (id)initWithSize:(size_t)size  
			 error:(NSError **)error;


- (id)initWithNSData:(NSData *)nsData
			mimeType:(NSString *)mimeType
			   error:(NSError **)error;


- (id)initWithContentsOfURL:(NSURL *)aURL  
					  error:(NSError **)error;

- (id)initWithNXData:(nx_data_fwd_t)nxData  
			   error:(NSError **)error;

- (id)initWithNXDataWrapper:(NXDataWrapper *)nxDataWrapper  
					  error:(NSError **)error;


- (NSData *)dataWithError:(NSError **)error;

- (const void *)bytes;
- (size_t)length;

- (BOOL)getBytes:(void *)buffer 
		  length:(size_t *)length
		   error:(NSError **)error;

- (NSString *)mimeType;
- (NSString *)mimeTypeWithError:(NSError **)error;
- (BOOL)setMIMEType:(NSString *)mimeType
			  error:(NSError **)error;

- (NSString *)description;
- (NSString *)descriptionWithError:(NSError **)error;
- (BOOL)setDescription:(NSString *)description
				 error:(NSError **)error;


- (NSURL *)sourceURL;
- (NSURL *)sourceURLWithError:(NSError **)error;
- (BOOL)setSourceURL:(NSURL *)sourceURL
			   error:(NSError **)error;

- (NXFileStatWrapper *)sourceStat;
- (NXFileStatWrapper *)sourceStatWithError:(NSError	**)error;
- (BOOL)setSourceStat:(NXFileStatWrapper *)fileStatWrapper
				error:(NSError **)error;

- (BOOL)getNXData:(nx_data_fwd_t *)nxData
			error:(NSError **)error;

- (NSDate *)fileCreationDate;
- (NSDate *)fileModificationDate;
- (NSDate *)fileAccessDate;
- (unsigned long long)fileSize;

@end
